/*
  Licensed under the GNU Public License.
  Copyright (C) 1998 by Thomas M. Vier, Jr. All Rights Reserved.

  wipe is free software.
  See LICENSE for more information.
*/

/* conditional includes */

#ifdef HAVE_MEMORY_H
# include <memory.h>
#endif

#ifdef HAVE_FCNTL_H
# include <fcntl.h>
#else
# undef HAVE_FCNTL
#endif

#ifndef HAVE_FCNTL
# ifndef HAVE_LOCKF
#  ifdef HAVE_FLOCK
#   ifdef HAVE_SYS_FILE_H
#    include <sys/file.h>
#   else
#    undef HAVE_FLOCK
#   endif
#  endif
# endif
#endif

#ifdef HAVE_GETOPT_H
# include <getopt.h>
#endif

#ifdef HAVE_DIRENT_H
# include <dirent.h>
# define NAMLEN(dirent) strlen((dirent)->d_name)
#else
# define dirent direct
# define NAMLEN(dirent) (dirent)->d_namlen)
# ifdef HAVE_SYS_NDIR_H
#  include <sys/ndir.h>
# endif
# ifdef HAVE_SYS_DIR_H
#  include <sys/dir.h>
# endif
# ifdef HAVE_NDIR_H
#  include <ndir.h>
# endif
#endif

/* conditional defines */

#ifdef STAT_MACROS_BROKEN
/* just in case, so we don't unlink a directory */
/* we don't currently handle broken stat macros */
# define unlink(x) remove(x)
#endif

#ifndef HAVE_UNLINK
# define unlink(x) remove(x)
#endif

#ifndef HAVE_RMDIR
# define rmdir(x) remove(x)
#endif

#undef SYNC
#ifdef HAVE_FDATASYNC
# define SYNC

/* for some reason, this proto is needed for glibc2, even though
 * it's already in unistd.h, cuz for some reason, it wasn't being
 * properly included
 */
extern int fdatasync  (int __fildes)    ; /* copied from unistd.h */

#elif HAVE_FSYNC
# define SYNC
#endif

#ifdef SYNC
# ifndef HAVE_FDATASYNC
#  ifdef HAVE_FSYNC
#   define fdatasync(fd) fsync(fd)
#  endif
# endif
#endif

#undef LOCKABLE
#ifdef HAVE_FCNTL
# define LOCKABLE
#elif HAVE_LOCKF
# define LOCKABLE
#elif HAVE_FLOCK
# define LOCKABLE
#endif

/* structs */

/* percent data */
struct complete
{
  float complete;               /* percentage of wipe completion             */
  float increment;              /* add to percentage after each pass         */
  unsigned int display:1;       /* show percentage                           */
  unsigned int reported:1;      /* set if percentage was reported            */
};

/* command line options */
struct opt
{
  unsigned int true_random:1;   /* use /dev/urandom if true                  */
  unsigned int verbose:2;       /* verbose level                             */
  unsigned int recursion:1;     /* traverse directories if true              */
  unsigned int zero:1;          /* zero-out file                             */
  unsigned int force:1;         /* force wipes -- override interaction       */
  unsigned int delete:1;        /* remove targets if true                    */
  unsigned int custom:1;        /* wipe file with user specified byte        */
  unsigned int random:1;        /* perform random passes if true             */
  unsigned int stream:1;        /* if true, use a stream of (P)RN's          */
  unsigned int interactive:1;   /* prompt for each file                      */
  unsigned int random_loop:5;   /* how many times to loop the random passes  */
  unsigned int wipe_multiply:5; /* how many times to loop all 35 passes      */
};

/* prototypes */

#ifdef LOCKABLE
int lock(void);
#endif

void help(int code, FILE *out);
void badopt(int c);
void usage(int code, FILE *out);
void show_copy(void);
void show_war(void);

void percent_init(void);

char getrn_char(void);
char gettrn_char(void);
unsigned long int getseed(void);
char getprn_char(void);
void getprn_char_stream_init(void);
char getprn_char_stream(void);
void prandom_pass(int passes);
void prandom_stream_pass(int passes);

void random_pass(int passes);
void random_stream_pass(int passes);
void write_pass(const unsigned char wipe_byte);
void write3_pass(const unsigned char wipe_byte1, const unsigned char wipe_byte2, const unsigned char wipe_byte3);

char rand_safe_char(void);
void rename_str(char str[], size_t len);
int wipename(void);

int destroy(char file[]);
int zero(void);
int wipe(void);

int getfiledata(char *file);
int drill_down(char dirname[]);
int check_file(char file[]);

int main(int argc, char **argv);
